/*
 * Usage.cpp
 *
 *  Created on: May 18, 2012
 *      Author: andrew
 */

#include "Usage.h"
#include <fstream>
#include <iostream>
using namespace std;

Usage::Usage(int in_num_cores)
{
	num_cores = in_num_cores;

	// Prepare arrays for CPU statistics
	cores_stats = new core_stats[num_cores];

	// Open /proc/stat
	//ifstream stat_file;
	//stat_file.open (STAT_NAME);
	stat_file = new std::ifstream(STAT_NAME);
	if (!stat_file->is_open())
		exit(-1);
}

Usage::~Usage()
{
	// Close /proc/stat
	stat_file->close();
	delete stat_file;

	// Trash allocated memory
	delete [] cores_stats;
}

void
Usage::recompute(void)
{
	char pattern[5];
	std::string buffer;
	char* buffer_cstr;
	const char delim[] = " ";

	unsigned int nice_time;
	unsigned int user_time;
	unsigned int idle_time;
	unsigned int system_time;

	unsigned int nice_time_passed;
	unsigned int user_time_passed;
	unsigned int idle_time_passed;
	unsigned int system_time_passed;
	unsigned int total_time_passed;

	// By default, assume that no cores have been updated and that the
	// CPU is disabled (its usage is 0)
	for (unsigned int i = 0; i < num_cores; i++)
	{
		cores_stats[i].updated = false;
		cores_stats[i].usage = 0;
	}

	// Seek back to start of file
	// HACK: this should technically use 'seekg' a la
	// stat_file->seekg(0, std::ios::beg);
	// though it seems we can only get this to perform reliably when we
	// close and reopen the stat file
	stat_file->close();
	stat_file->open(STAT_NAME);
	if ( stat_file->rdstate() != 0 )
		printf("\n\n\n I broke! \n\n\n");
	// Retrieve each line of file
	while (std::getline(*stat_file, buffer))
	{
		// Check to see if this line matches cpu name
		for (unsigned int i = 0; i < num_cores; i++)
		{
			sprintf(pattern, "cpu%d", i);
			// printf("@@Looking for pattern: %d\n", i);

			if (buffer.find(pattern) != std::string::npos)
			{
				// printf ("@@Matched pattern for core %d\n", i);

				// Transfer buffer to C string
				buffer_cstr = new char[buffer.size() + 1];
				strcpy (buffer_cstr, buffer.c_str());

				// Split the line for the CPU into separate times
				strtok(buffer_cstr, delim);
				user_time = atoi(strtok(NULL, delim));
				nice_time = atoi(strtok(NULL, delim));
				system_time = atoi(strtok(NULL, delim));
				idle_time = atoi(strtok(NULL, delim));

				// Calculate the time passed since the last check
				user_time_passed = user_time - cores_stats[i].user_time_last;
				nice_time_passed = nice_time - cores_stats[i].nice_time_last;
				system_time_passed = system_time - cores_stats[i].system_time_last;
				idle_time_passed = idle_time - cores_stats[i].idle_time_last;

				total_time_passed = user_time_passed + nice_time_passed +
						system_time_passed + idle_time_passed;

				// Usage is non-idle time divided by total time
				// the usage calculation was put into an if statement because a few of the cores did not
				if (total_time_passed != 0)
					cores_stats[i].usage = ((float) (total_time_passed - idle_time_passed) /
											(float)(total_time_passed) * 100.0);

				// Store current time measurements
				cores_stats[i].user_time_last = user_time;
				cores_stats[i].nice_time_last = nice_time;
				cores_stats[i].system_time_last = system_time;
				cores_stats[i].idle_time_last = idle_time;

				cores_stats[i].updated = true;
				//break;
			}
		}
	}
	stat_file->close();
}

float
Usage::get_usage(int core_id)
{
	return cores_stats[core_id].usage;
}
